@@ -11,7 +11,7 @@ use File::Spec;
 use File::HomeDir ();
 use Fcntl;
 
-our $VERSION = 0.26;
+our $VERSION = 0.27;
 
 BEGIN {
     if ($^O =~ /Win32/i) {
@@ -36,6 +36,7 @@ my $properties = {
     'show_tainted'   => 1,
     'show_weak'      => 1,
     'use_prototypes' => 1,
+    'output'         => 'stderr',
     'return_value'   => 'dump',       # also 'void' or 'pass'
     'colored'        => 'auto',       # also 0 or 1
     'caller_info'    => 0,
@@ -79,6 +80,7 @@ my $properties = {
         -class => [ \&_class ],
     },
 
+    _output          => *STDERR,     # used internally
     _current_indent  => 0,           # used internally
     _linebreak       => \$BREAK,     # used internally
     _seen            => {},          # used internally
@@ -163,7 +165,7 @@ sub _print_and_return {
     my ($item, $dump, $p) = @_;
 
     if ( $p->{return_value} eq 'pass' ) {
-        print STDERR $dump . $/;
+        print { $p->{_output} } $dump . $/;
 
         my $ref = ref $item;
         if ($ref eq 'ARRAY') {
@@ -180,11 +182,11 @@ sub _print_and_return {
         }
     }
     elsif ( $p->{return_value} eq 'void' ) {
-        print STDERR $dump . $/;
+        print { $p->{_output} } $dump . $/;
         return;
     }
     else {
-        print STDERR $dump . $/ unless defined wantarray;
+        print { $p->{_output} } $dump . $/ unless defined wantarray;
         return $dump;
     }
 }
@@ -214,7 +216,7 @@ sub _data_printer {
           or ($p->{colored} eq 'auto'
               and (exists $ENV{ANSI_COLORS_DISABLED}
                    or $wantarray
-                   or not -t *STDERR
+                   or not -t $p->{_output}
                   )
           )
     ) {
@@ -256,7 +258,7 @@ sub _p {
     delete $p->{_reftype}; # abort override
 
     # globs don't play nice
-    $ref = 'GLOB' if "$item" =~ /=GLOB\([^()]+\)$/;
+    $ref = 'GLOB' if "$item" =~ /GLOB\([^()]+\)$/;
 
 
     # filter item (if user set a filter for it)
@@ -305,7 +307,9 @@ sub SCALAR {
         $string .= colored($$item, $p->{color}->{'number'});
     }
     else {
-        $string .= colored(qq["$$item"], $p->{color}->{'string'});
+        my $val = $$item;
+        $val =~ s/\0/\\0/g;
+        $string .= colored(qq["$val"], $p->{color}->{'string'});
     }
 
     $string .= ' ' . colored('(TAINTED)', $p->{color}->{'tainted'})
@@ -752,6 +756,43 @@ sub _merge {
                     }
                 }
             }
+            elsif ($key eq 'output') {
+                my $out = $p->{output};
+                my $ref = ref $out;
+
+                $clone->{output} = $out;
+
+                my %output_target = (
+                     stdout => *STDOUT,
+                     stderr => *STDERR,
+                );
+
+                my $error;
+                if (!$ref and exists $output_target{ lc $out }) {
+                    $clone->{_output} = $output_target{ lc $out };
+                }
+                elsif ( ( $ref and $ref eq 'GLOB')
+                     or (!$ref and \$out =~ /GLOB\([^()]+\)$/)
+                ) {
+                    $clone->{_output} = $out;
+                }
+                elsif ( !$ref or $ref eq 'SCALAR' ) {
+                    if( open my $fh, '>>', $out ) {
+                        $clone->{_output} = $fh;
+                    }
+                    else {
+                        $error = 1;
+                    }
+                }
+                else {
+                    $error = 1;
+                }
+
+                if ($error) {
+                    Carp::carp 'Error opening custom output handle.';
+                    $clone->{_output} = $output_target{ 'stderr' };
+                }
+            }
             else {
                 $clone->{$key} = $p->{$key};
             }
@@ -889,12 +930,32 @@ L<JSON>, or whatever. CPAN is full of such solutions!
 
 Once you load Data::Printer, the C<p()> function will be imported
 into your namespace and available to you. It will pretty-print
-into STDERR whatever variabe you pass to it.
+into STDERR (or any other output target) whatever variabe you pass to it.
+
+=head2 Changing output targets
+
+By default, C<p()> will be set to use STDERR. As of version 0.27, you
+can set up the 'output' property so Data::Printer outputs to
+several different places:
+
+=over 4
+
+=item * C<< output => 'stderr' >> - Standard error. Same as *STDERR
+
+=item * C<< output => 'stdout' >> - Standard output. Same as *STDOUT
+
+=item * C<< output => $filename >> - Appends to filename.
+
+=item * C<< output => $file_handle >> - Appends to opened handle
+
+=item * C<< output => \$scalar >> - Appends to that variable's content
+
+=back
 
 =head2 Return Value
 
 If for whatever reason you want to mangle with the output string
-instead of printing it to STDERR, you can simply ask for a return
+instead of printing it, you can simply ask for a return
 value:
 
   # move to a string
@@ -968,8 +1029,8 @@ Don't fancy colors? Disable them with:
 
 By default, 'colored' is set to C<"auto">, which means Data::Printer
 will colorize only when not being used to return the dump string,
-nor when STDERR is being piped. If you're not seeing colors, try
-forcing it with:
+nor when the output (default: STDERR) is being piped. If you're not
+seeing colors, try forcing it with:
 
   use Data::Printer colored => 1;
 
@@ -1018,6 +1079,8 @@ customization options available, as shown below (with default values):
       caller_info    => 0,       # include information on what's being printed
       use_prototypes => 1,       # allow p(%foo), but prevent anonymous data
       return_value   => 'dump',  # what should p() return? See 'Return Value' above.
+      output         => 'stderr',# where to print the output. See
+                                 # 'Changing output targets' above.
 
       class_method   => '_data_printer', # make classes aware of Data::Printer
                                          # and able to dump themselves.
@@ -1502,7 +1565,7 @@ previous dumping function.
 
 If, however, you want a really unified approach where you can easily
 flip between debugging outputs, use L<Any::Renderer> and its plugins,
-like L<< Any::Renderer::Data::Printer|https://github.com/kmcgrath/Any-Renderer-Data-Printer >>.
+like L<Any::Renderer::Data::Printer>.
 
 =head2 Printing stack traces with arguments expanded using Data::Printer
 
@@ -1571,6 +1634,8 @@ with patches, bug reports, wishlists, comments and tests. They are
 
 =over 4
 
+=item * Allan Whiteford
+
 =item * Árpád Szász
 
 =item * brian d foy
@@ -1583,6 +1648,8 @@ with patches, bug reports, wishlists, comments and tests. They are
 
 =item * Damien Krotkine (dams)
 
+=item * Denis Howe
+
 =item * Dotan Dimet
 
 =item * Eden Cardim (edenc)
@@ -1591,6 +1658,12 @@ with patches, bug reports, wishlists, comments and tests. They are
 
 =item * Fernando Corrêa (SmokeMachine)
 
+=item * Fitz Elliott
+
+=item * J Mash
+
+=item * Jesse Luehrs (doy)
+
 =item * Kartik Thakore (kthakore)
 
 =item * Kevin McGrath (catlgrep)
@@ -1615,6 +1688,8 @@ with patches, bug reports, wishlists, comments and tests. They are
 
 =item * Tatsuhiko Miyagawa (miyagawa)
 
+=item * Tim Heaney (oylenshpeegul)
+
 =item * Torsten Raudssus (Getty)
 
 =item * Wesley Dal`Col (blabos)
